#include "config.h"
#include <stdio.h>
#include <stdlib.h> 
#include <ctype.h> 
#include <string.h>
#include "pngin.h"
#include "pngout.h"
#include "ks0108.h"
#include "text.h"

int Configuration::isblank(char val) {
    if (val<33) return 1; else return 0;
}

char Configuration::splitvals(char* in, char* name, char* val) {
    //These routines need buffer overrun checking, but that's not here yet.
    //Should be allright though, as long as name>in and val>in
    char *currchar;
    char *outchar;
    char delim=0;
//    printf("Splitvals: splitting '%s'.\n",in);
    currchar=in;
    while (isblank(*currchar) && *currchar!=0) currchar++;
    if (*currchar==0) return(0);
    if (*currchar=='#') return(0);
    outchar=name;
    //Copy the name-part of the string (one word till either a blank or = or :
    while (!isblank(*currchar) && (*currchar!='=') && (*currchar!=':')) {
	*outchar=tolower(*currchar);
	currchar++;
	outchar++;
    }
    *outchar=0;
//    printf("Splitvals: name='%s'.\n",name);
    //Skip past the whitespace and/or =/:
    while (isblank(*currchar) || (*currchar=='=') || (*currchar==':')) {
	if (!isblank(*currchar)) delim=*currchar;
	currchar++;
    }
//    printf("Splitvals: delim=0x%x.\n",delim);
    //Copy the value
    outchar=val;
    while (*currchar!=0) {
	*outchar=*currchar;
	currchar++;
	outchar++;
    }
    *outchar=0;
    //Delete trailing whitespace
    while (isblank(*outchar) && (outchar>val)) {
//	printf("Backstepping... '%s'\n",val);
	*outchar=0;
	outchar--;
    }
//    printf("Splitvals: value='%s'.\n",val);
    return delim;
}

#define STATE_NOTHING 0
#define STATE_INPUTP 1
#define STATE_OUTPUTP 2

Configuration::Configuration(FILE *ConfigFile) {
    int state=STATE_NOTHING;
    char buffer[STRLEN_CONFIGLINELEN];
    char name[STRLEN_CONFIGLINELEN];
    char value[STRLEN_CONFIGLINELEN];
    char delim;
    int iplugno=-1;
    int oplugno=-1;
    
    printf("Creating new configuration.\n");
    
    while (fgets(buffer,STRLEN_CONFIGLINELEN-1,ConfigFile)) {
	delim=splitvals(buffer,name,value);
	if (strlen(name)>0) {
	    if (delim==':') {
		printf("New plugin, type %s, plugin name %s\n",name, value);
		if (strcmp(name,"input")==0) {
		    state=STATE_INPUTP;
		    iplugno++;
		    if (strcmp(value,"pngin")==0) ipluglist[iplugno]=new pngin; else
		    if (strcmp(value,"text")==0) ipluglist[iplugno]=new text; else
		     {
			printf("Error: no such input plugin: '%s'\n",value);
			exit(0);
		    }
		} else if (strcmp(name,"output")==0) {
		    state=STATE_OUTPUTP;
		    oplugno++;
		    if (strcmp(value,"pngout")==0) opluglist[oplugno]=new pngout; else
		    if (strcmp(value,"ks0108")==0) opluglist[oplugno]=new ks0108; else
		     {
			printf("Error: no such output plugin: '%s'\n",value);
			exit(0);
		    }
		} else {
		    printf("Plugin type unsupported: %s\n",name);
		    exit(0);
		}
	    } else if (delim=='=') {
		if (state==STATE_INPUTP) {
		    ipluglist[iplugno]->parse_inputline(name,value);
		} else if (state==STATE_OUTPUTP) {
		    opluglist[oplugno]->parse_inputline(name,value);
		} else {
		    printf("Select a plugin first!\n");
		}
	    }
	}
    }
}

void Configuration::display() {
    int i;
    gdImagePtr img;
    gdImagePtr *tmpimage;
    printf("Creating output image (%ix%i)",opluglist[0]->get_width(),opluglist[0]->get_height());
    img=gdImageCreateTrueColor(opluglist[0]->get_width(),opluglist[0]->get_height());
    gdImageAlphaBlending(img,1);
    printf("Building image.\n");
    for (i=0; i<MAXIPLUGIN; i++) {
	if (ipluglist[i]!=NULL) {
	    printf("Processing input plugin %i...\n",i);
	    tmpimage=ipluglist[i]->get_output();
	    printf("Got output. Merging into end-image @ %i,%i.\n",ipluglist[i]->GetX(),ipluglist[i]->GetY());
	    gdImageCopy(img,*tmpimage,ipluglist[i]->GetX(),ipluglist[i]->GetY(),0,0,gdImageSX(*tmpimage),gdImageSY(*tmpimage));
	}
    }
    for (i=0; i<MAXOPLUGIN; i++) {
	if (opluglist[i]!=NULL) {
	    printf("Processing output plugin %i...\n",i);
	    opluglist[i]->output(img);
	}
    }
    printf("Processing done.\n");
}
