#include <gd.h>
#include <string.h> 
#include <stdlib.h> 
#include <sys/io.h>
#include <unistd.h>
#include "ks0108.h"

#define LCD_DELAY 100

ks0108::ks0108(){
    printf("ks0108 obj created.\n");
    height=64;
    width=128;
    pport=0x378;
}

int ks0108::get_height(){
    printf("getheight\n");
    return height;
}

int ks0108::get_width(){
    printf("getwidth\n");
    return width;
}

int ks0108::parse_inputline(char *name, char *value) {
    printf("ks0108 - %s: %s\n",name,value);
    if (strcmp(name, "height")==0) {
	height=atoi(value);
	return(0);
    }
    if (strcmp(name, "width")==0) {
	width=atoi(value);
	return(0);
    }
    if (strcmp(name, "pport")==0) {
	pport=strtol(value, (char **)NULL, 16);
	return(0);
    }
    return(outputplugin::parse_inputline(name,value));
}

// Base
//Data pins
// Base+2
#define LCD_RS 1<<3
#define LCD_CS1 1<<1
#define LCD_CS2 1<<2
#define LCD_E 1<<0
#define INVMASK 11

void ks0108::senddata(int chip, int dc, int data) {
    int i=0;
    if ((chip&1)==0) i|=LCD_CS1;
    if ((chip&2)==0) i|=LCD_CS2;
    if (dc==1) i|=LCD_RS;
    outb(data,pport);
    outb(i^ INVMASK,pport+2);
    usleep(LCD_DELAY);
    outb((i+LCD_E)^ INVMASK,pport+2);
    usleep(LCD_DELAY);
    outb(i^ INVMASK,pport+2);
//    usleep(LCD_DELAY);
}

int ks0108::output(gdImagePtr img) {
    int x,y,b,p1,p2;
    printf("Requesting acces for pport @ 0x%x\n",pport);
    if (ioperm(pport,3,1)!=0) {
	perror("ks0108 (ioperm for pport)");
	return(0);
    }
    printf("Initing lcd...\n");
    senddata(3,0,0x3f);
    senddata(3,0,0x40);
    senddata(3,0,0xb0);
    senddata(3,0,0xc0);
    printf("Sending data...\n");
    for (y=0; y<8; y++) {
	senddata(3,0,0xb8 + y);
	for (x=0; x<64; x++) {
	    p1=0; p2=0;
	    for (b=0; b<8; b++) {
		p1=p1>>1;
		p2=p2>>1;
		if (gdImageGreen(img,gdImageGetPixel(img,x,y*8+b))>128) p1+=128;
		if (gdImageGreen(img,gdImageGetPixel(img,x+64,y*8+b))>128) p2+=128;
	    }
	    printf("x=%i y=%i",x,y);
	    senddata(1,1,p1);
	    senddata(2,1,p2);
	}
    }
}
